﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic

Imports System.IO
Imports System.Diagnostics
Imports System.Threading

Namespace Mutation
    Friend Class Program
        Private Shared ran As New Random(2)
        Shared Sub Main(ByVal args() As String)
            Try
                Console.WriteLine(vbLf & "Begin super-simple mutation testing demo" & vbLf)

                Dim originalSourceFile = "..\..\Class1-Original.vb"
                Dim mutatedSourceFile = "..\..\..\MathLib\Class1.vb"
                Dim mutantProject = "..\..\..\MathLib\MathLib.vbproj"
                Dim testProject = "..\..\..\TestMutation\TestMutation.vbproj"
                Dim testExecutable = "..\..\..\TestMutation\bin\Debug\TestMutation.exe"
                Dim devenv = "C:\Program Files (x86)\Microsoft Visual Studio 9.0\Common7\IDE\devenv.exe"

                Console.WriteLine("Determining possible mutation positions in MathLib.dll source code")
                Dim positions = GetMutationPositions(originalSourceFile)
                Dim numberMutants = 2

                For i = 0 To numberMutants - 1
                    Console.WriteLine(vbLf & "=============")
                    Console.WriteLine("Mutant # " & i)
                    Console.WriteLine(vbLf & "Selecting random mutation position")
                    Dim randomPosition = positions(ran.Next(0, positions.Count))
                    Console.WriteLine("Creating mutant source code file")
                    CreateMutantSource(originalSourceFile, randomPosition, mutatedSourceFile)

                    Try
                        Console.WriteLine(vbLf & "Begin build mutant MathLib.dll")
                        BuildMutant(mutantProject, devenv)
                        Console.WriteLine(vbLf & "Rebuilding test harness with mutated DLL")
                        BuildTestProject(testProject, devenv)
                        Console.WriteLine(vbLf & "Testing mutant DLL and writing results to log file")
                        Console.WriteLine("Check log to make sure at least one test failure generated!")
                        TestMutant(testExecutable)
                    Catch
                        Console.WriteLine("Invalid mutant. Aborting.")
                        Continue For
                    End Try
                    Console.WriteLine("=============")
                Next i

                Console.WriteLine(vbLf & "Mutation test run complete")
                'Console.ReadLine();
            Catch ex As Exception
                Console.WriteLine("Fatal: " & ex.Message)
                Console.ReadLine()
            End Try
        End Sub ' Main()

        Private Shared Function GetMutationPositions(ByVal originalSourceFile As String) As List(Of Integer)
            Dim sr = File.OpenText(originalSourceFile)
            Dim ch = 0
            Dim pos = 0
            Dim list As New List(Of Integer)
            ch = sr.Read()
            Do While ch <> -1
                If ChrW(ch) = ">"c OrElse ChrW(ch) = "<"c Then
                    list.Add(pos)
                End If
                pos += 1
                ch = sr.Read()
            Loop
            sr.Close()
            Return list
        End Function ' GetMutationPositions

        Private Shared Sub CreateMutantSource(ByVal originalSourceFile As String,
                                              ByVal mutatePosition As Integer,
                                              ByVal mutatedSourceFile As String)
            Dim ifs As New FileStream(originalSourceFile, FileMode.Open)
            Dim sr As New StreamReader(ifs)
            Dim ofs As New FileStream(mutatedSourceFile, FileMode.Create)
            Dim sw As New StreamWriter(ofs)
            Dim currPos = 0
            Dim currChar As Integer

            currChar = sr.Read()
            Do While currChar <> -1
                If currPos = mutatePosition Then
                    If ChrW(currChar) = "<"c Then
                        sw.Write(">"c)
                        Console.WriteLine("Mutating < to > at position " & currPos)
                    ElseIf ChrW(currChar) = ">"c Then
                        sw.Write("<"c)
                        Console.WriteLine("Mutating > to < at position " & currPos)
                    Else
                        sw.Write(ChrW(currChar))
                    End If
                Else
                    sw.Write(ChrW(currChar))
                End If

                currPos += 1
                currChar = sr.Read()
            Loop

            sw.Close()
            ofs.Close()
            sr.Close()
            ifs.Close()
        End Sub ' CreateMutant

        Private Shared Sub BuildMutant(ByVal mutantSolution As String, ByVal devenv As String)
            Dim psi As New ProcessStartInfo(devenv, mutantSolution & " /rebuild")
            Dim p As New Process

            p.StartInfo = psi
            p.Start()
            Do While p.HasExited = False
                System.Threading.Thread.Sleep(400)
                Console.WriteLine("Waiting for mutant build to complete . . ")
            Loop
            p.Close()
        End Sub ' BuildMutant()

        Private Shared Sub BuildTestProject(ByVal testProject As String, ByVal devenv As String)
            Dim psi As New ProcessStartInfo(devenv, testProject & " /rebuild")
            Dim p As New Process

            p.StartInfo = psi
            p.Start()
            Do While p.HasExited = False
                System.Threading.Thread.Sleep(500)
                Console.WriteLine("Waiting for test project build to complete . . ")
            Loop
            p.Close()
        End Sub ' BuildTestProject()

        Private Shared Sub TestMutant(ByVal testExecutable As String)
            Dim psi As New ProcessStartInfo(testExecutable)
            Dim p As New Process
            p.StartInfo = psi
            p.Start()
            Do While p.HasExited = False
                System.Threading.Thread.Sleep(200)
                'Console.WriteLine("Running test cases . . ");
            Loop
            p.Close()

        End Sub ' TestMutant()

    End Class ' Program
End Namespace ' ns